//  This file is part of foo_ir2.
//
//  foo_ir2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  foo_ir2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with foo_ir2. If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include "sdk/foobar2000/sdk/foobar2000.h"
#include "sdk/foobar2000/helpers/helpers.h"

#include <strsafe.h>

#include "helper.h"
#include "serial/serialwnd.h"
#include "serialqueue.h"

#define WU_SERIALPORTEVENT WM_USER
#define WU_UPDATEWINDOW WM_USER+1

////////////
// CONFIG //
////////////
static const GUID foo_ir2_preference_guid = { 0xd33c78cc, 0x9ecd, 0x41a7, { 0xb7, 0x00, 0x65, 0x1c, 0x22, 0xf2, 0x34, 0xdb } };

static const GUID cfg_foo_ir2_enabled_guid = { 0xf7047f33, 0xa595, 0x4b15, { 0x8a, 0xe1, 0x70, 0x62, 0x9e, 0x4f, 0x9c, 0x79 } };
cfg_bool cfg_ir2_enabled( cfg_foo_ir2_enabled_guid, false );
static const GUID cfg_baudrate_guid = { 0x66d9e027, 0xa2bb, 0x4feb, { 0xbf, 0x2b, 0x5b, 0x72, 0x6b, 0xed, 0xf8, 0x45 } };
cfg_int cfg_baudrate( cfg_baudrate_guid, CSerial::EBaud9600 );
static const GUID cfg_initstring_guid = { 0xca0e104c, 0xd972, 0x4530, { 0x90, 0x71, 0x95, 0x7c, 0x2a, 0xb5, 0x37, 0x1a } };
cfg_string cfg_initstring( cfg_initstring_guid, "IR" );
static const GUID cfg_comport_guid = { 0xca0e104c, 0xd972, 0x4530, { 0x90, 0x71, 0x95, 0x7c, 0x2a, 0xb5, 0x37, 0x1b } };
cfg_string cfg_comport( cfg_comport_guid, "COM1" );
static const GUID cfg_parity_guid = { 0xca0e104c, 0xd972, 0x4530, { 0x90, 0x71, 0x95, 0x7c, 0x2a, 0xb5, 0x37, 0x1c } };
cfg_int cfg_parity( cfg_parity_guid, CSerial::EParNone );
static const GUID cfg_databits_guid = { 0xca0e104c, 0xd972, 0x4530, { 0x90, 0x71, 0x95, 0x7c, 0x2a, 0xb5, 0x37, 0x1d } };
cfg_int cfg_databits( cfg_databits_guid, CSerial::EData8 );
static const GUID cfg_stopbits_guid = { 0xca0e104c, 0xd972, 0x4530, { 0x90, 0x71, 0x95, 0x7c, 0x2a, 0xb5, 0x37, 0x1e } };
cfg_int cfg_stopbits( cfg_stopbits_guid, CSerial::EStop1 );
static const GUID cfg_flowcontrol_guid = { 0xca0e104c, 0xd972, 0x4530, { 0x90, 0x71, 0x95, 0x7c, 0x2a, 0xb5, 0x37, 0x1f } };
cfg_int cfg_flowcontrol( cfg_flowcontrol_guid, CSerial::EHandshakeOff );
static const GUID cfg_setdtr_guid = { 0xf7047f33, 0xa595, 0x4b15, { 0x8a, 0xe1, 0x70, 0x62, 0x9e, 0x4f, 0x9c, 0x78 } };
cfg_bool cfg_setdtr( cfg_setdtr_guid, true );
static const GUID cfg_setrts_guid = { 0xf7047f33, 0xa595, 0x4b15, { 0x8a, 0xe1, 0x70, 0x62, 0x9e, 0x4f, 0x9c, 0x77 } };
cfg_bool cfg_setrts( cfg_setrts_guid, true );
static const GUID cfg_codelength_guid = { 0xf7047f33, 0xa595, 0x4b15, { 0x8a, 0xe1, 0x70, 0x62, 0x9e, 0x4f, 0x9c, 0x76 } };
cfg_int cfg_codelength( cfg_codelength_guid, 4 );
static const GUID cfg_startupdelay_guid = { 0xf7047f33, 0xa595, 0x4b15, { 0x8a, 0xe1, 0x70, 0x62, 0x9e, 0x4f, 0x9c, 0x75 } };
cfg_int cfg_startupdelay( cfg_startupdelay_guid, 50 );
static const GUID cfg_delay_guid = { 0xf7047f33, 0xa595, 0x4b15, { 0x8a, 0xe1, 0x70, 0x62, 0x9e, 0x4f, 0x9c, 0x74 } };
cfg_int cfg_delay( cfg_delay_guid, 200 );

#define MAXCODELENGTH 8
static struct t_ircodes
{
	char play_code[MAXCODELENGTH];
	char pause_code[MAXCODELENGTH];
	char stop_code[MAXCODELENGTH];
	char next_code[MAXCODELENGTH];
	char prev_code[MAXCODELENGTH];
	char nextrand_code[MAXCODELENGTH];
	char backw_code[MAXCODELENGTH];
	char forw_code[MAXCODELENGTH];
	char volup_code[MAXCODELENGTH];
	char voldown_code[MAXCODELENGTH];
	char mute_code[MAXCODELENGTH];
} def_ircodes;
static const GUID cfg_ircodes_guid = { 0xca0e104c, 0xd972, 0x4530, { 0x90, 0x71, 0x95, 0x7c, 0x2a, 0xb5, 0x37, 0x19 } };
cfg_struct_t<t_ircodes> cfg_ircodes( cfg_ircodes_guid, def_ircodes );
/////////////////////////

// serial port stuff
static CSerialWnd g_SerialPort;
static HWND g_hMessageWnd;
static bool g_fOKReceived;
static CSerialQueue<MAXCODELENGTH, char> g_SerialQueue;
static DWORD g_dwTimeOfLastCommand;

// points to the remote control button we are currently learning
static enum
{
	nothing, play, pause, stop, next, nextrandom, previous, forward, backward, volumeup, volumedown, mute
} g_eLearning;

class foo_ir2
{
public:
	foo_ir2();
	~foo_ir2();

	static LRESULT CALLBACK WndProc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp );
	static void OpenSerialPort();
	static void CloseSerialPort();
};
static foo_ir2* g_foo_ir2 = NULL;

////////////////////////
// PREFERENCES WINDOW //
////////////////////////
static HWND g_hPrefWnd;
class ir2_config : public preferences_page
{
public:
	friend foo_ir2;
	static void LearnCheck();

	static BOOL CALLBACK ProcWindowMsgs( HWND hWnd, UINT uMsg, WPARAM wp, LPARAM lp );
	HWND create( HWND parent )
	{
		g_hPrefWnd = uCreateDialog( IDD_CONFIG_IR2, parent, ProcWindowMsgs, 0 );
		return g_hPrefWnd;
	}

	virtual const char* get_name() { return "ir2"; }
	virtual GUID get_guid() { return foo_ir2_preference_guid; }
	virtual GUID get_parent_guid() { return guid_tools; }
	virtual bool reset_query() { return true; }
	virtual void reset()
	{
		foo_ir2::CloseSerialPort();
		cfg_ir2_enabled = false;
		cfg_baudrate = CSerial::EBaud9600;
		cfg_initstring = "IR";
		cfg_comport = "COM1";
		cfg_parity = CSerial::EParNone;
		cfg_databits = CSerial::EData8;
		cfg_stopbits = CSerial::EStop1;
		cfg_flowcontrol = CSerial::EHandshakeOff;
		cfg_setdtr = true;
		cfg_setrts = true;
		cfg_codelength = 4;
		cfg_startupdelay = 50;
		cfg_delay = 200;

		t_ircodes ircodes;
		memset( &ircodes, 0, sizeof(ircodes) );
		cfg_ircodes = ircodes;
	}
	virtual bool get_help_url( pfc::string_base &p_out )
	{
		p_out = "http://www.nonoo.hu/projects/foo_ir2/";
		return true;
	}

private:
	static void UpdateWindow();
};
static preferences_page_factory_t<ir2_config> g_config_ir2;

//////////////
// INITQUIT //
//////////////
class ir2_initquit : public initquit
{
public:
	virtual void on_init()
	{ 
		g_foo_ir2 = new foo_ir2();
	}
	virtual void on_quit()
	{
		SAFE_DELETE( g_foo_ir2 );
	}
};
static initquit_factory_t<ir2_initquit> g_initquit_ir2;
//////////////

DECLARE_COMPONENT_VERSION(
	"ir2 Infrared Remote Control",
	"0.1.2",
	"ir2 Infrared Remote Control\nhttp://www.nonoo.hu/projects/foo_ir2/\n\n" \
	"Copyright (C) 2008 Nonoo <nonoo@nonoo.hu>"
);
