//  This file is part of foo_ir2.
//
//  foo_ir2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  foo_ir2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with foo_ir2. If not, see <http://www.gnu.org/licenses/>.

#pragma once

template <size_t MaxQueueSize, class T>
class CSerialQueue
{
public:
	CSerialQueue();

	void push( T input );
	T pop();
	size_t length();
	void fill( T input = 0);
	void erase( T input = 0 );
	T* getdata();
	void setsize( size_t nNewSize );
	size_t getsize();

private:
	void shiftleft();

	T m_Data[MaxQueueSize];
	unsigned int m_nLastItem;
	T m_PopReturn;
	size_t m_nQueueSize;
};

////////////////

template <size_t MaxQueueSize, class T>
CSerialQueue<MaxQueueSize, T>::CSerialQueue()
{
	m_nLastItem = 0;
	m_nQueueSize = MaxQueueSize;
}

template <size_t MaxQueueSize, class T>
void CSerialQueue<MaxQueueSize, T>::push( T input )
{
	if( m_nLastItem == m_nQueueSize )
		shiftleft();
	m_Data[m_nLastItem++] = input;
}

template <size_t MaxQueueSize, class T>
T CSerialQueue<MaxQueueSize, T>::pop()
{
	m_PopReturn = m_Data[0];
	shiftleft();
	return m_PopReturn;
}

template <size_t MaxQueueSize, class T>
size_t CSerialQueue<MaxQueueSize, T>::length()
{
	return m_nLastItem;
}

template <size_t MaxQueueSize, class T>
void CSerialQueue<MaxQueueSize, T>::fill( T input )
{
	for( unsigned int i = 0; i < m_nQueueSize; i++ )
		m_Data[i] = input;
}

template <size_t MaxQueueSize, class T>
void CSerialQueue<MaxQueueSize, T>::erase( T input )
{
	m_nLastItem = 0;
	fill( input );
}

template <size_t MaxQueueSize, class T>
T* CSerialQueue<MaxQueueSize, T>::getdata()
{
	return (T*)&m_Data;
}

template <size_t MaxQueueSize, class T>
void CSerialQueue<MaxQueueSize, T>::setsize( size_t nNewSize )
{
	if( nNewSize > MaxQueueSize )
		m_nQueueSize = MaxQueueSize;
	else
		m_nQueueSize = nNewSize;
}

template <size_t MaxQueueSize, class T>
size_t CSerialQueue<MaxQueueSize, T>::getsize()
{
	return m_nQueueSize;
}

template <size_t MaxQueueSize, class T>
void CSerialQueue<MaxQueueSize, T>::shiftleft()
{
	if( m_nLastItem == 0 )
		return;

	if( m_nLastItem == 1 )
	{
		m_nLastItem = 0;
		return;
	}

	for( unsigned int i = 1; i < m_nLastItem + 1; i++ )
		m_Data[i-1] = m_Data[i];

	m_nLastItem--;
}
